//
//  MyDocument.m
//  MatrixMaster
//
//  Created by Jeremy Clark on 2/26/07.
//  Copyright __MyCompanyName__ 2007 . All rights reserved.
//

#import "MyDocument.h"
#import "Value.h"
#import "Result.h"
#import "Parameter.h"
#import "Case.h"


@interface MyDocument (PrivateMethods)

-(void)clickCell:(id)sender;
-(void)toggleResultState:(id)sender;
-(void)selectCell:(id)sender;
-(void)updateCombinations:(id)sender;
-(Case *)caseForValues:(NSArray *)values;
-(void)updateCaseSelection;

@end

static int kTextHeightFactor = 13;

@implementation MyDocument

- (id)init 
{
    self = [super init];
    if (self != nil) {
        // initialization code
		_defaultButtonColor = [[NSColor lightGrayColor] retain];
		
    }
    return self;
}

- (void) dealloc {
	[_defaultButtonColor release];
	[super dealloc];
}




- (NSString *)windowNibName 
{
    return @"MyDocument";
}


- (void)windowControllerDidLoadNib:(NSWindowController *)windowController 
{
    [super windowControllerDidLoadNib:windowController];
    // user interface preparation code

}


- (void)awakeFromNib{
	
	NSSortDescriptor * sd = [[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES];
	[selectedValueController setSortDescriptors:[NSArray arrayWithObject:sd]];
	[parameterController setSortDescriptors:[NSArray arrayWithObject:sd]];
	[sd release];
	
	
	//Setup the scroll view
	[m_matrixScrollView setHasHorizontalScroller:YES];
	[m_matrixScrollView setHasVerticalScroller:YES];
	[m_matrixScrollView setBorderType:NSBezelBorder];
	[m_matrixScrollView setBackgroundColor:[NSColor lightGrayColor]];
	[m_matrixScrollView setDrawsBackground:YES];
	[m_matrixScrollView setDocumentView:[[FlippedView alloc] initWithFrame:NSMakeRect(0,0,150,120)]];
	[[m_matrixScrollView documentView] addSubview:batchControlView];
	
	
	//Create the cell prototype for the result matrix
	m_resultButtonCell = [[[NSButtonCell alloc] init] retain];
	[m_resultButtonCell setControlSize:NSMiniControlSize];
	[m_resultButtonCell setButtonType:NSSwitchButton];
	[m_resultButtonCell setAllowsMixedState:YES];
	[m_resultButtonCell setImagePosition:NSNoImage];
	[m_resultButtonCell setBackgroundColor:_defaultButtonColor];
	[m_resultButtonCell setCellAttribute:NSCellLightsByGray to:1];
	[m_resultButtonCell setTitle:@""];
	[m_resultButtonCell setBordered:YES];
	[m_resultButtonCell setTarget:self];
	[m_resultButtonCell setAction:@selector(clickCell:)];
	[m_resultMatrix setMenu:resultContextualMenu];
	[m_resultMatrix setAllowsEmptySelection:YES];
	[m_resultMatrix setMode:NSHighlightModeMatrix];
	[m_resultMatrix setPrototype:m_resultButtonCell];
	
	
	
	//Create the cell prototype for the label matrices
	NSTextFieldCell * labelCell = [[[NSTextFieldCell alloc] init] autorelease];
	[labelCell setBackgroundColor:[NSColor grayColor]];
	[labelCell setDrawsBackground:YES];
	[labelCell setLineBreakMode:NSLineBreakByTruncatingTail];
	[labelCell setFont:[NSFont labelFontOfSize:10.]];
	[yLabelMatrix setPrototype:labelCell];
	[xLabelMatrix setPrototype:labelCell];
	
	[yLabelMatrix setMode:NSListModeMatrix];
	[yLabelMatrix setTarget:self];
	[yLabelMatrix setAction:@selector(selectRow:)];
	[xLabelMatrix setMode:NSListModeMatrix];
	[xLabelMatrix setTarget:self];
	[xLabelMatrix setAction:@selector(selectColumn:)];
	
	
	//Set the testcase properties to defaults
	[self setTestcaseName:@"Sample Testcase"];
	[self setExpectedResult:@"This testcase should be awesome. More awesomer than you."];
	[self setSteps:@"1. Eat food\r2. You know what comes next"];
	
	
	[m_plottedCombinationsTable setRowHeight:([[parameterController arrangedObjects] count] * 13)];
	
	
}



- (void)clickCell:(id)sender
{
	
	NSLog(@"%i", [[sender selectedCell] state]);
	
	int currentMode = [_mode intValue];
	if (currentMode == 0){
		[self toggleResultState:sender];
	}else if (currentMode == 1){
		[[m_resultMatrix cells] makeObjectsPerformSelector:@selector(setBackgroundColor:) withObject:_defaultButtonColor];
		[self selectCell:[m_resultMatrix selectedCell]];
	}
	
}



- (void)toggleResultState:(id)sender
{
	


	id selectedCell = [sender selectedCell];
	id representedObject = [selectedCell representedObject];
	[representedObject setValue:[NSNumber numberWithInt:[selectedCell state]] forKey:@"status"];
	
	if ([[representedObject valueForKey:@"status"] intValue] < 0){ //fail
		[selectedCell setBackgroundColor:[NSColor redColor]];
		[self setPlottedFailCount:[NSNumber numberWithInt:[[self plottedFailCount] intValue] + 1]];
	}
	
	if ([[representedObject valueForKey:@"status"] intValue] > 0){ //pass
		[selectedCell setBackgroundColor:[NSColor greenColor]];
		[self setPlottedPassCount:[NSNumber numberWithInt:[[self plottedPassCount] intValue] + 1]];
		[self setPlottedFailCount:[NSNumber numberWithInt:[[self plottedFailCount] intValue] - 1]];
	}
	
	if ([[representedObject valueForKey:@"status"] intValue] == 0){
		[selectedCell setBackgroundColor:_defaultButtonColor];
		[self setPlottedPassCount:[NSNumber numberWithInt:[[self plottedPassCount] intValue] - 1]];
	}
	
	[self setPlottedExecutedCount:[NSNumber numberWithInt:([[self plottedPassCount] intValue] + [[self plottedFailCount] intValue])]];
	[self willChangeValueForKey:@"plottedExecutedPercentage"];
	[self didChangeValueForKey:@"plottedExecutedPercentage"];
	
	//[sender highlightCell:YES atRow:[sender selectedRow]  column:[sender selectedColumn]];

	
	[sender setNeedsDisplay:YES];

}





- (IBAction)selectCell:(id)sender
{
	
	[xLabelMatrix deselectAllCells];
	[xLabelMatrix selectCellAtRow:[m_resultMatrix selectedColumn] column:0];
	[yLabelMatrix deselectAllCells];
	[yLabelMatrix selectCellAtRow:[m_resultMatrix selectedRow] column:0];

	[self updateCaseSelection];

}







- (IBAction)selectColumn:(id)sender{
	
	int currentMode = [_mode intValue];
	if (currentMode == 0){
	
	
	
	}else if (currentMode == 1){
		

		int firstCellToSelect;
		int lastCellToSelect;
		int topRowToSelect = 0;
		int bottomRowToSelect = [m_resultMatrix numberOfRows] - 1;
		
		//Calculate the left and right columns of the selection
		NSArray * selectedColumnCells = [sender selectedCells];
		int selectedColumnsCount = [selectedColumnCells count];
		if (selectedColumnsCount == 1){ //single column selected
			_leftmostSelectedColumn = [sender selectedRow];
			_rightmostSelectedColumn = [sender selectedRow];
		}else{ //It's a multiple column selection. Determine if it's direction
			if ([[sender cellAtRow:([sender selectedRow] + 1) column:0] isHighlighted]){ //it's a 'right to left' selection
				_leftmostSelectedColumn = [sender selectedRow];
				_rightmostSelectedColumn = ([sender selectedRow] + selectedColumnsCount) - 1;
			}else{ //it's a 'left to right' selection
				_rightmostSelectedColumn = [sender selectedRow];
				_leftmostSelectedColumn = ([sender selectedRow] - selectedColumnsCount) + 1;
			}
		}
		
		
		//Update the intersection correction if it's turned on
		if ([_intersectSelections boolValue]){
			topRowToSelect = _topSelectedRow;
			bottomRowToSelect = _bottomSelectedRow;
		}else{
			[yLabelMatrix deselectAllCells];
		}
		
		NSLog(@"Top Row to Select: %i", topRowToSelect);
		NSLog(@"Bottom Row to Select: %i", bottomRowToSelect);
		
		
		//Calculate the first and last cells to select
		firstCellToSelect = (_leftmostSelectedColumn) + (topRowToSelect * [m_resultMatrix numberOfColumns]);
		lastCellToSelect = _rightmostSelectedColumn + (bottomRowToSelect * [m_resultMatrix numberOfColumns]);
		
		[m_resultMatrix deselectAllCells];
		[m_resultMatrix setSelectionFrom:firstCellToSelect to:lastCellToSelect  anchor:firstCellToSelect highlight:YES];
		[m_resultMatrix setNeedsDisplay:YES];
	}

	[self updateCaseSelection];
	

}




- (IBAction)selectRow:(id)sender{
	
	int currentMode = [_mode intValue];
	if (currentMode == 0){
	
	
	
	}else if (currentMode == 1){
		
		int firstCellToSelect;
		int lastCellToSelect;
		int leftColumnToSelect = 0;
		int rightColumnToSelect = [m_resultMatrix numberOfColumns] - 1;
		
		
		//Calculate the bottom and top selected rows
		NSArray * selectedRowCells = [sender selectedCells];
		int selectedRowCount = [selectedRowCells count];
		if (selectedRowCount == 1){ //single column selected
			_topSelectedRow = [sender selectedRow];
			_bottomSelectedRow = [sender selectedRow];
		}else{ //It's a multiple column selection. Determine if it's direction
			if ([[sender cellAtRow:([sender selectedRow] + 1) column:0] isHighlighted]){ //it's a 'bottom to top' selection
				_topSelectedRow = [sender selectedRow];
				_bottomSelectedRow = ([sender selectedRow] + selectedRowCount) - 1;
			}else{ //it's a 'top to bottom' selection
				_bottomSelectedRow = [sender selectedRow];
				_topSelectedRow = ([sender selectedRow] - selectedRowCount) + 1;
			}
		}
		
		//Update the intersection correction if it's turned on
		if ([_intersectSelections boolValue]){
			leftColumnToSelect = _leftmostSelectedColumn;
			rightColumnToSelect = _rightmostSelectedColumn;
		}else{
			[xLabelMatrix deselectAllCells];
		}
		

		//Calculate the first and last cell to select
		firstCellToSelect = (_topSelectedRow * [m_resultMatrix numberOfColumns]) + (leftColumnToSelect);
		lastCellToSelect = (_bottomSelectedRow * [m_resultMatrix numberOfColumns]) + (rightColumnToSelect);
		
		[m_resultMatrix deselectAllCells];
		[m_resultMatrix setSelectionFrom:firstCellToSelect to:lastCellToSelect  anchor:firstCellToSelect highlight:YES];
		[m_resultMatrix setNeedsDisplay:YES];
	}
	
	[self updateCaseSelection];
	
}




- (void)updateCaseSelection
{

	NSMutableArray * selectedCases = [NSMutableArray array];
	NSEnumerator * selectedCells = [[m_resultMatrix selectedCells] objectEnumerator];
	id aCell;
	while (aCell = [selectedCells nextObject]){
		Case * aCase = [[aCell representedObject] valueForKey:@"case"];
		[selectedCases addObject:aCase];
	}
	[m_casesController setSelectedObjects:selectedCases];
	[m_plottedCombinationsTable scrollRowToVisible:[m_casesController selectionIndex]];
}






- (IBAction)removeValue:(id)sender
{
	NSArray * selectedValues = [selectedValueController valueForKey:@"selectedObjects"];
	NSEnumerator * valueEnum = [selectedValues objectEnumerator];
	id nextValue;
	while (nextValue = [valueEnum nextObject]){
		[[self managedObjectContext] deleteObject:nextValue];
		[[self managedObjectContext] refreshObject:nextValue mergeChanges:YES];
	}
}


//Method to get the current mouse location
- (id)resultCellAtCurrentMouseLocation
{
	NSPoint matrixPoint = [m_resultMatrix convertPoint:[[m_resultMatrix window] mouseLocationOutsideOfEventStream] fromView:[[m_resultMatrix window] contentView]];
	int row;
	int column;
	[m_resultMatrix getRow:&row column:&column forPoint:matrixPoint];
	return [m_resultMatrix cellAtRow:row column:column];
}




- (IBAction)showCaseDetails:(id)sender
{

	//Get the result for the current cell
	Case * aCase = [[[self resultCellAtCurrentMouseLocation] representedObject] valueForKey:@"case"];
	[m_casesController setSelectedObjects:[NSArray arrayWithObject:aCase]];

}



- (IBAction)editSelectedCases:(id)sender
{

	//Show the edit forms window as a modal form
	[[NSApplication sharedApplication] runModalForWindow:editCaseFormPanel];
	
	[editCaseFormPanel orderOut:self];

}




- (IBAction)closeEditPanel:(id)sender
{

	
    if ([[sender title] isEqualToString:@"Cancel"]){ //Do nothing
		NSLog(@"Cancel");
    }else if([[sender title] isEqualToString:@"Submit"]){ //Update each selected case's expected result
		NSString * expectedResultString = [m_expectedResultFormTextView string];
		NSString * stepsString = [m_stepsTextView string];
		NSEnumerator * selectedCasesEnum = [[m_casesController selectedObjects] objectEnumerator];
		id nextCase;
		while (nextCase = [selectedCasesEnum nextObject]){
			[nextCase setValue:expectedResultString forKey:@"expectedResult"];
			[nextCase setValue:stepsString forKey:@"steps"];
		}
		[self updateMatrix:self];
    }

		
    [[NSApplication sharedApplication] stopModal];


}




- (IBAction)logRadar:(id)sender
{
	//Get the result for the current cell
	Result * result = [[self resultCellAtCurrentMouseLocation] representedObject];
	
	NSUserDefaults * sharedDefaults = [NSUserDefaults standardUserDefaults];
	NSString * radarDescription = [NSString stringWithFormat:@"*SUMMARY\n%@\n\n* STEPS TO REPRODUCE\n%@\n\n* RESULTS\nExpected: %@\n\nActual: \n\n* BUILD\n%@", [NSString stringWithFormat:@"%@ - Failed", [self testcaseName]], [self steps], [self expectedResult], [[buildController selection] valueForKey:@"name"]];
	NSString * escapedRadarDescription = [[radarDescription componentsSeparatedByString:@"\""] componentsJoinedByString:@"\\\""];
	NSString * duration = [NSString stringWithString:@"3"];


	NSString * radarProblemTitle = [NSString stringWithFormat:@"%@ - Failed", [self testcaseName]];
	NSDictionary * errorDict;

	NSString * radarNewProblemScriptString = [NSString stringWithFormat:@"tell application \"Radar\"\nset newProblemID to NewProblem componentName \"%@\" componentVersion \"%@\" description \"%@\" problemTitle \"%@\" reproducibilityCode %i classCode %i doSave %i formType %i originator \"%@\"\nignoring application responses\nset resultCode to SetProblemData problemID newProblemID columnNames \"featureTextCaseID=%@\"\nend ignoring\nend tell\nreturn newProblemID", 
		[sharedDefaults valueForKey:@"radarComponentName"], 
		[sharedDefaults valueForKey:@"radarComponentVersion"], 
		escapedRadarDescription,  
		radarProblemTitle, 
		([[sharedDefaults valueForKey:@"radarReproducibility"] intValue] + 1), 
		([[sharedDefaults valueForKey:@"radarClassification"] intValue] + 1), 
		[[sharedDefaults valueForKey:@"radarDoSave"] intValue], 
		([[sharedDefaults valueForKey:@"radarFormType"] intValue] + 1), 
		[sharedDefaults valueForKey:@"radarOriginator"],
		duration];


	NSAppleScript * newRadarProblemScript = [[NSAppleScript alloc] initWithSource:radarNewProblemScriptString];
	NSAppleEventDescriptor * newRadarProblemExecution = [newRadarProblemScript executeAndReturnError:&errorDict];

	if (!newRadarProblemExecution){
		NSRunAlertPanel(nil, [errorDict valueForKey:@"NSAppleScriptErrorMessage"] , @"Cancel", @"OK", nil);
	}else {
		[[resultsController selection] setValue:[newRadarProblemExecution stringValue] forKey:@"radar"];
	}
	
}


- (IBAction)exportToRallyCSV:(id)sender{

	//Initialize the string
	NSString *rallyCSVString = [NSMutableString stringWithString:@"\"Work Product\",Name,Description,\"Validation Expected Result\",type"];
	
	
	//Then, create a case for each one
	NSEnumerator * allCasesEnum = [[m_casesController selectedObjects] objectEnumerator];
	id nextCase;
	NSString * workProductString = [[self displayName] stringByReplacingOccurrencesOfString:@"-" withString:@": "];
	while (nextCase = [allCasesEnum nextObject]){
		NSString * tcName = [nextCase valueForKey:@"rallyName"];
		NSString * tcSteps = [nextCase valueForKey:@"steps"];
		NSString * tcExpectedResult = [nextCase valueForKey:@"expectedResult"];
		rallyCSVString = [rallyCSVString stringByAppendingString:[NSString stringWithFormat:@"\r%@,\"%@\",\"%@\",\"%@\",Acceptance", workProductString, tcName,tcSteps,tcExpectedResult]];
	}
	
	
	[rallyCSVString writeToFile:@"/export.csv"atomically:YES];
	
	[[NSTask launchedTaskWithLaunchPath:@"/usr/bin/open" arguments:[NSArray arrayWithObject:@"/export.csv"]] launch];

	
}
	








- (BOOL)validateMenuItem:(NSMenuItem *)item {
    
	
	if ([item action] == @selector(logRadar:)){
		Result * result = [[self resultCellAtCurrentMouseLocation] representedObject];
		if ([[result valueForKey:@"status"] intValue] < 0){
			return YES;
		}else{
			return NO;
		}
    }
		
	return YES;
   
}


	
	
- (void)updateCombinations:(id)sender
{
	
	//First get all combinations
	NSMutableArray * allValuesArray = [NSMutableArray array];
	NSEnumerator * allParams = [[parameterController arrangedObjects] objectEnumerator];
	id nextParam;
	while (nextParam = [allParams nextObject]){
		NSArray * valuesForParam = [[nextParam valueForKey:@"values"] allObjects];
		[allValuesArray addObject:valuesForParam];
	}
	NSArray * allCombinations = [self combineList:allValuesArray index:0];
	
	//Then, create a case for each one
	NSEnumerator * allCombinationsEnum = [allCombinations objectEnumerator];
	id nextCombination;
	while (nextCombination = [allCombinationsEnum nextObject]){
		[self caseForValues:nextCombination];
	}
	
	//set the row height of the combination table
	[m_plottedCombinationsTable setRowHeight:([[parameterController arrangedObjects] count] * 13)];
	

}



- (IBAction)updateMatrix:(id)sender
{
	
	[self updateCombinations:sender];
	
	NSArray *xCombos = [self valueCombinationsForAxis:@"x"];
	NSArray *xComboStrings = [self stringsForValueCombinations:xCombos];
	NSArray *yCombos = [self valueCombinationsForAxis:@"y"];
	NSArray *yComboStrings = [self stringsForValueCombinations:yCombos];
	
	
	//NOW CREATE THE MATRICES
	int x;
	int y;
	
	
	//Y LABEL MATRIX
	//Set up the yLabel cell height, by getting the count of parameter sets to be plotted on the y axis and multiplying the text height factor
	[yLabelMatrix renewRows:0 columns:0];
	float yLabelCellHeight = 0;
	float yLabelMatrixHeight = 0;
	if ([yCombos count] > 0){
		yLabelCellHeight = [[yCombos objectAtIndex:0] count] * kTextHeightFactor;
		[yLabelMatrix setCellSize:NSMakeSize(150,yLabelCellHeight)];
		for (y=0;y<[yCombos count];y++){
			[yLabelMatrix addRow];
			[[yLabelMatrix cellAtRow:y column:0] setObjectValue:[yComboStrings objectAtIndex:y]];
			yLabelMatrixHeight = yLabelMatrixHeight + yLabelCellHeight + 2;
		}
		[yLabelMatrix setFrameSize:NSMakeSize(150,yLabelMatrixHeight)];
		[yLabelMatrix setFrameOrigin:NSMakePoint(0,[batchControlView frame].size.height + 5)];
	}
	[[m_matrixScrollView documentView] addSubview:yLabelMatrix];

	
	
	
	//X LABEL MATRIX
	//Set up the xLabel cell width, by getting the count of parameter sets to be plotted on the y axis and multiplying the text height factor
	[xLabelMatrix renewRows:0 columns:0];
	[xLabelMatrix setBoundsRotation:-90]; 
	float xLabelCellWidth = 0;
	float xLabelMatrixWidth = 0;
	if ([xCombos count] > 0){
		xLabelCellWidth = [[xCombos objectAtIndex:0] count] * kTextHeightFactor;
		[xLabelMatrix setCellSize:NSMakeSize([batchControlView frame].size.height, xLabelCellWidth)];
		for (x=0;x<[xCombos count];x++){
			[xLabelMatrix addRow];
			[[xLabelMatrix cellAtRow:x column:0] setObjectValue:[xComboStrings objectAtIndex:x]];
			xLabelMatrixWidth = xLabelMatrixWidth + xLabelCellWidth + 2;
		}
		[xLabelMatrix setFrameSize:NSMakeSize(xLabelMatrixWidth, 110)];
		[xLabelMatrix setFrameOrigin:NSMakePoint([batchControlView frame].size.width + 2,0)];
	}
	[[m_matrixScrollView documentView] addSubview:xLabelMatrix];
	
	
	//Now set the size of the document view in the scroll view based on the label matrices
	float docHeight = yLabelMatrixHeight + 115;
	float docWidth = xLabelMatrixWidth + 150;
	[[m_matrixScrollView documentView] setFrameSize:NSMakeSize(docWidth,docHeight)];
	
	
	//RESULT MATRIX
	int passedTestsCount = 0;
	int failedTestsCount = 0;
	[m_plottedCombinationsController setContent:NULL];
	int totalTestsCount = [xCombos count]*[yCombos count];
	
	[[m_matrixScrollView documentView] addSubview:m_resultMatrix];
	[m_resultMatrix renewRows:0 columns:0];
	[m_resultMatrix setCellSize:NSMakeSize(xLabelCellWidth,yLabelCellHeight)];
	[m_resultMatrix setFrameSize:NSMakeSize(xLabelMatrixWidth, yLabelMatrixHeight)];
	[m_resultMatrix setFrameOrigin:NSMakePoint([xLabelMatrix frame].origin.x,[yLabelMatrix frame].origin.y)];
	for (x = 0;x<[xCombos count];x++){
		
		[m_resultMatrix addColumn];

		for (y = 0;y<[yCombos count];y++){
			
			if ((x == 0) && (y > 0)){

				[m_resultMatrix addRow];
				
			}
			
			//First, get an array that is the "UNION" of the x and y combinations for a particular cell
			NSArray * cellValueArray = [[[xCombos objectAtIndex:x] allObjects] arrayByAddingObjectsFromArray:[[yCombos objectAtIndex:y] allObjects]];
			
			//Get the total value for those values
			//int totalValue = [self totalValueForValues:cellValueArray];
			
			//Get the case for that value combination
			Case * relatedCase = [self caseForValues:cellValueArray];
			
			//Now, see if a result exists for that case and build, and if it doesn't then create one
			id currentBuild = [[buildController valueForKey:@"selectedObjects"] objectAtIndex:0];
			Result * result = [self resultForBuild:currentBuild andCase:relatedCase];
			
			//Now create a cell at "x,y" in the matrix, and hook it up to the result
			[m_resultMatrix makeCellAtRow:y column:x];
			id newCell = [m_resultMatrix cellAtRow:y column:x];
			
			//set the state
			[newCell setState:[[result valueForKey:@"status"] intValue]]; 
			
			//Set the tooltip
			NSString * comboString = [NSString stringWithFormat:@"%@\r%@", [xComboStrings objectAtIndex:x], [yComboStrings objectAtIndex:y]];
			NSString * expectedResultString = [NSString stringWithFormat:@"\rExpected Result:\r%@", [relatedCase valueForKey:@"expectedResult"]];
			[m_resultMatrix setToolTip:[NSString stringWithFormat:@"%@\r%@", comboString, expectedResultString] forCell:newCell];
			//[m_resultMatrix setToolTip:[NSString stringWithFormat:@"%@\r%@\r\r%i", comboString, expectedResultString, totalValue] forCell:newCell];
						
			
			//Now display the corresponding color for the cell's Result
			[newCell setRepresentedObject:result];
			if ([[[newCell representedObject] valueForKey:@"status"] intValue] < 0){//Fail
				[newCell setBackgroundColor:[NSColor redColor]]; 
				failedTestsCount++;
			}
			
			if ([[[newCell representedObject] valueForKey:@"status"] intValue] > 0){ //Pass
				[newCell setBackgroundColor:[NSColor greenColor]];
				passedTestsCount++;
			}
			
			if ([[[newCell representedObject] valueForKey:@"status"] intValue] == 0) [newCell setBackgroundColor:_defaultButtonColor]; //Not Executed
			
		}
		
	}
	
	[self setPlottedTestCount:[NSNumber numberWithInt:totalTestsCount]];
	[self setPlottedFailCount:[NSNumber numberWithInt:failedTestsCount]];
	[self setPlottedPassCount:[NSNumber numberWithInt:passedTestsCount]];
	[self setPlottedExecutedCount:[NSNumber numberWithInt:((failedTestsCount + passedTestsCount))]];
	[self willChangeValueForKey:@"plottedExecutedPercentage"];
	[self didChangeValueForKey:@"plottedExecutedPercentage"];
	
	
	[m_matrixScrollView setNeedsDisplay:YES];

	
			
}



-(int)totalValueForValues:(NSArray *)values
{
	
	int totalValue = 0;
	NSEnumerator * valueEnum = [values objectEnumerator];
	id nextValue;
	while (nextValue = [valueEnum nextObject]){
		totalValue = totalValue + [[nextValue valueForKey:@"name"] intValue];
	}
	return totalValue;
	
}





-(NSArray *)valueCombinationsForAxis:(NSString *)anAxis{
	
	NSMutableArray * valueSetsList = [NSMutableArray array];
	
	NSString *fetchRequestName;
	if ([anAxis isEqualTo:@"x"]){
		fetchRequestName = @"xParameters";
	}else if ([anAxis isEqualTo:@"y"]){
		fetchRequestName = @"yParameters";
	}
	
	//Now fetch the parameters to plot for the indicated axis
	NSError *error;
	NSFetchRequest * fr = [[[self managedObjectModel] fetchRequestTemplateForName:fetchRequestName] copy];
	NSArray * params = [[self managedObjectContext] executeFetchRequest:fr error:&error];

	
	//For each parameter, let's add it's 'valuesToPlot' property sorted by name to the valueSetsList - then we'll pass this into the combiner
	NSEnumerator * paramEnum = [params objectEnumerator];
	id nextParam;
	while (nextParam = [paramEnum nextObject]){
		[[self managedObjectContext] refreshObject:nextParam mergeChanges:YES]; //we need to refresh so that the valuesToPlot property issues a new fetch
		NSArray * sortedValues = [[nextParam valueForKey:@"valuesToPlot"] sortedArrayUsingDescriptors:[selectedValueController sortDescriptors]];
		[valueSetsList addObject:sortedValues];
	}
	
	//Now process the value sets list
	if ([valueSetsList count] == 0){
		return [NSArray array];
	}else{
		return [self combineList:valueSetsList index:0];
	}
	
	[fr release];

}
	






-(NSArray *)stringsForValueCombinations:(NSArray *)valueCombinations
{
	NSMutableArray * valueCombinationStrings = [[NSMutableArray alloc] init];
	
	NSEnumerator * valueComboEnum = [valueCombinations objectEnumerator];
	id nextValueCombo;
	while (nextValueCombo = [valueComboEnum nextObject]){
		NSMutableString * valueComboString = [[NSMutableString alloc] init];
		NSEnumerator * valueEnum = [nextValueCombo objectEnumerator];
		id nextValue;
		while (nextValue = [valueEnum nextObject]){
			NSString * parameterName = [nextValue valueForKeyPath:@"parameter.name"];
			[valueComboString appendFormat:@"%@: %@\r", parameterName, [nextValue valueForKey:@"name"]];
		}
		[valueComboString deleteCharactersInRange:NSMakeRange([valueComboString length] - 1 ,1)];
		
		[valueCombinationStrings addObject:valueComboString];
	
	}
	
	return valueCombinationStrings;
	
}





- (Case *)caseForValues:(NSArray *)values
{

	NSArray *filteredArray = [m_casesController valueForKey:@"arrangedObjects"];
	NSEnumerator * valuesEnum = [values objectEnumerator];
	id nextValue;
	while (nextValue = [valuesEnum nextObject]){
		filteredArray = [[NSArray arrayWithArray:filteredArray] filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"ANY values.objectID == %@", [nextValue objectID]]];
	}
	
	if ([filteredArray count] == 0){
		Case * newCase = [NSEntityDescription insertNewObjectForEntityForName:@"Case" inManagedObjectContext:[self managedObjectContext]];
		[newCase setValue:@"Expected Result" forKey:@"expectedResult"];
		[newCase setValue:@"Steps" forKey:@"steps"];
		NSEnumerator * valuesEnum = [values objectEnumerator];
		Value * nextValue;
		while (nextValue = [valuesEnum nextObject]){
			[newCase addValuesObject:nextValue];
		}
		NSLog(@"new case created %@", [newCase description]);
		return newCase;
	}else{
		Case * existingCase = [filteredArray objectAtIndex:0];
		NSLog(@"existing case loaded %@", [existingCase description]);
		return existingCase;
	}

}





- (Result *)resultForBuild:(id)build andCase:(Case *)aCase
{
	
	
	NSArray *resultsArray = [resultsController valueForKey:@"arrangedObjects"];
	
	NSArray * filteredArray = [[NSArray arrayWithArray:resultsArray] filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"build == %@", build]];
	filteredArray = [[NSArray arrayWithArray:filteredArray] filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"case.objectID == %@", [aCase objectID]]];
	
	if ([filteredArray count] == 0){
		Result * result = [NSEntityDescription insertNewObjectForEntityForName:@"Result" inManagedObjectContext:[self managedObjectContext]];
		[result setValue:build forKey:@"build"];
		[result setValue:aCase forKey:@"case"];
		NSLog(@"new result created %@", [result description]);
		return result;
	}else{
		Result * result = [filteredArray objectAtIndex:0];
		NSLog(@"existing result loaded %@", [result description]);
		return result;
	}
	
	
}




- (NSArray *)combineList:(NSArray *)someList index:(int)index
{

	if (index == ([someList count])){
		return [NSArray arrayWithObject:[NSSet set]];
	}else{
		NSMutableArray * result = [[NSMutableArray alloc] init];
		//NSArray * listA = [[someList objectAtIndex:index] sortedArrayUsingDescriptors:[NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES] autorelease]]];
		NSArray * listA = [someList objectAtIndex:index];
		NSArray * listB = [self combineList:someList index:(index + 1)];
		Value * nextListAItem;
		id nextListBItem;
		NSEnumerator * listAEnum = [listA objectEnumerator];
		while (nextListAItem = [listAEnum nextObject]){
			NSEnumerator * listBEnum = [listB objectEnumerator];
			while (nextListBItem = [listBEnum nextObject]){
				NSMutableSet * setToAdd = [[NSMutableSet alloc] init];
				[setToAdd addObject:nextListAItem];
				[setToAdd unionSet:nextListBItem];
				[result addObject:setToAdd];
				[setToAdd release];
			}
			
		}
		return result;
	}
	
}



#pragma mark -
#pragma mark ACCESSOR METHODS


//Mode Properties
- (NSNumber *)mode
{
	return _mode;
}


- (void)setMode:(NSNumber *)num
{
	[num retain];
	[_mode release];
	_mode = num;
	NSLog(@"%i", [_mode intValue]);
	if ([_mode intValue] == 1){
		[[m_resultMatrix cells] makeObjectsPerformSelector:@selector(setBackgroundColor:) withObject:_defaultButtonColor];
		[m_resultMatrix setMode:NSListModeMatrix];
	}else{
		[m_resultMatrix setMode:NSHighlightModeMatrix];
		[self updateMatrix:self];
	}

}


- (NSString *)testcaseName
{
	return m_testcaseName;
}

- (void)setTestcaseName:(NSString *)name
{
	[m_testcaseName release];
	[name retain];
	m_testcaseName = name;
}

- (NSString *)expectedResult
{
	return m_expectedResult;
}

- (void)setExpectedResult:(NSString *)expectedResult
{
	[m_expectedResult release];
	[expectedResult retain];
	m_expectedResult = expectedResult;
}

- (NSString *)steps
{
	return m_steps;
}

	
- (void)setSteps:(NSString *)steps
{
	[m_steps release];
	[steps retain];
	m_steps = steps;
}
	


//Current plot state
- (NSNumber *)plottedTestCount
{
	return _plottedTestCount;
}

- (void)setPlottedTestCount:(NSNumber *)count
{
	[self willChangeValueForKey:@"plottedTestCount"];
	[_plottedTestCount release];
	[count retain];
	_plottedTestCount = count;
	[self didChangeValueForKey:@"plottedTestCount"];
}

- (NSNumber *)plottedPassCount
{
	return _plottedPassCount;
}

- (void)setPlottedPassCount:(NSNumber *)count
{
	[self willChangeValueForKey:@"plottedPassCount"];
	[_plottedPassCount release];
	[count retain];
	_plottedPassCount = count;
	[self didChangeValueForKey:@"plottedPassCount"];
}


- (NSNumber *)plottedFailCount
{
	return _plottedFailCount;
}

- (void)setPlottedFailCount:(NSNumber *)count
{
	[self willChangeValueForKey:@"plottedFailCount"];
	[_plottedFailCount release];
	[count retain];
	_plottedFailCount = count;
	[self didChangeValueForKey:@"plottedFailCount"];
}


- (NSNumber *)plottedExecutedCount
{
	return _plottedExecutedCount;
}


- (void)setPlottedExecutedCount:(NSNumber *)count
{
	[self willChangeValueForKey:@"plottedExecutedCount"];
	[_plottedExecutedCount release];
	[count retain];
	_plottedExecutedCount = count;
	[self didChangeValueForKey:@"plottedExecutedCount"];
}


- (NSNumber *)plottedExecutedPercentage
{
	return [NSNumber numberWithFloat:(([[self plottedExecutedCount] floatValue]/[[self plottedTestCount] floatValue]) * 100.)];
}






//Table view delegate methods
#pragma mark -
#pragma mark TableView Delegate Methods

- (NSString *)tableView:(NSTableView *)tv toolTipForCell:(NSCell *)cell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *)tc row:(int)row mouseLocation:(NSPoint)mouseLocation {
    if ([cell isKindOfClass:[NSTextFieldCell class]]) {
        if ([[cell attributedStringValue] size].width > rect->size.width) {
            return [cell stringValue];
		}
	} 
    return nil;
}






//SCRIPTING STUFF
#pragma mark -
#pragma mark SCRIPTING STUFF


- (void)insertValue:(id)value inPropertyWithKey:(NSString *)key
{
	
	if ([key isEqualTo:@"parameters"]){
		
	}

}




- (NSArray *)parameters
{
	NSArray * result;
	NSError * error;
	NSFetchRequest *fr = [[[NSFetchRequest alloc] init] autorelease];
	NSEntityDescription * param = [NSEntityDescription entityForName:@"Parameter" inManagedObjectContext:[self managedObjectContext]];
	[fr setEntity:param];
	result = [[self managedObjectContext] executeFetchRequest:fr error:&error];
	return result;
}




@end





